import math
import pdb

import astropy.units as u
import matplotlib.pyplot as plt
import numpy as np
from astropy.io import ascii
from astropy.table import Table
from desk import config
from matplotlib import rc

'''
Steve Goldman
Space Telescope Science Institute
May 17, 2018
sgoldman@stsci.edu

This script is for plotting the outputs of the sed_fitting script.
'''


def create_fig():
    """
    :return: Runs plotting script
    """
    # rc('text', usetex=True)
    # plt.rcParams['font.family'] = 'serif'
    # plt.rcParams['mathtext.fontset'] = 'dejavuserif'
    # plt.rcParams['text.usetex'] = True
    # plt.rcParams['text.latex.unicode'] = True

    full_path = str(__file__.replace('plotting_seds.py', ''))
    input_file = Table.read('fitting_plotting_outputs.csv')
    grid_dusty = Table.read(full_path + 'models/' + str(input_file['grid_name'][0]) + '_models.fits')

    def get_data(filename):
        """
        :param filename: filename of input data. Should be csv with Column 0: wavelength in um and Col 1: flux in Jy
        :return: two arrays of wavelength (x) and flux (y) in unit specified in config.py
        """
        table = ascii.read(filename, delimiter=',')
        table.sort(table.colnames[0])
        x = np.array(table.columns[0])
        y = np.array(table.columns[1])
        if config.output['output_unit'] == 'Wm^-2':
            y = y * u.Jy
            y = y.to(u.W / (u.m * u.m), equivalencies=u.spectral_density(x * u.um))
        elif config.output['output_unit'] == 'Jy':
            pass
        else:
            raise ValueError("Unit in config.py not 'Wm^-2' or 'Jy'")
        return x, np.array(y)

    # setting axes
    if len(input_file) == 1:
        fig, ax1 = plt.subplots(1, 1, sharex=True, sharey=True, figsize=(8, 5))
    elif len(input_file) == 2:
        fig, axs = plt.subplots(2, 1, sharex=True, sharey=True, figsize=(8, 10))
    elif len(input_file) == 3:
        fig, axs = plt.subplots(3, 1, sharex=True, sharey=True, figsize=(8, 10))
    else:
        fig, axs = plt.subplots(math.ceil(len(input_file) / 3), 3, sharex=True, sharey=True, figsize=(8, 10))
        axs = axs.ravel()

    for counter, target in enumerate(input_file):
        # gets data for plotting
        target_name = (target['target_name']).replace('.csv', '').replace('_', ' ')
        x_data, y_data = get_data(target['data_file'])
        x_model, y_model = grid_dusty[target['index']]
        x_model = x_model[np.where(y_model != 0)]
        y_model = y_model[np.where(y_model != 0)] * input_file[counter]['norm']
        if config.output['output_unit'] == 'Wm^-2':
            axislabel = "log $\lambda$ F$_{\lambda}$ (W m$^{-2}$)"
        elif config.output['output_unit'] == 'Jy':
            y_model = y_model * u.W / (u.m * u.m)
            y_model = y_model / ((x_model * u.um).to(u.Hz, equivalencies=u.spectral()))
            y_model = y_model.to(u.Jy).value
            axislabel = "log F$_{\lambda}$ (Jy)"
        else:
            raise ValueError("Unit in config.py not 'Wm^-2' or 'Jy'")

        # logscale
        x_model = np.log10(x_model)
        y_model = np.log10(y_model)
        x_data = np.log10(x_data)
        y_data = np.log10(y_data)

        # out = Table((x_model,y_model), names=('wave', 'lamflam'))
        # out.write('crystalline_fit.csv', format='csv')
        # pdb.set_trace()
        # plotting
        if len(input_file) == 1:
            ax1.set_xlim(-0.99, 2.49)
            ax1.set_ylim(np.median(y_model) - 2, np.median(y_model) + 2)
            ax1.scatter(x_data, y_data, c='blue', label='data')
            ax1.plot(x_model, y_model, c='k', linewidth=0.5, linestyle='--', zorder=2, label='model')
            ax1.annotate(target_name.replace('-', r'\textendash'), (0.07, 0.85), xycoords='axes fraction', fontsize=14)
            ax1.get_xaxis().set_tick_params(which='both', direction='in', labelsize=15)
            ax1.get_yaxis().set_tick_params(which='both', direction='in', labelsize=15)
            ax1.set_xlabel('log $\lambda$ ($\mu m$)', labelpad=10)
            ax1.set_ylabel("log $\lambda$ F$_{\lambda}$ " + "(W m$^{-2}$)", labelpad=10)
        else:
            axs[counter].set_xlim(-0.99, 2.49)
            axs[counter].set_ylim(np.median(y_model) - 2, np.median(y_model) + 2)
            axs[counter].plot(x_model, y_model, c='k', linewidth=0.4, linestyle='--', zorder=2)
            axs[counter].scatter(x_data, y_data, c='blue')
            axs[counter].annotate(target_name.replace('-', r'\textendash'), (0.7, 0.8), xycoords='axes fraction',
                                  fontsize=14)
            axs[counter].get_xaxis().set_tick_params(which='both', direction='in', labelsize=15)
            axs[counter].get_yaxis().set_tick_params(which='both', direction='in', labelsize=15)
            axs[counter].set_xlabel('log $\lambda$ ($\mu m$)', labelpad=10)
            axs[counter].set_ylabel(axislabel, labelpad=10)

        # pdb.set_trace()
    plt.subplots_adjust(wspace=0, hspace=0)
    fig.savefig('output_sed.png', dpi=200, bbox_inches='tight')


if __name__ == '__main__':
    create_fig()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `dusty_evolved_star_kit` package."""


import unittest
from click.testing import CliRunner

import desk
from desk import sed_fit
# from dusty_evolved_star_kit import cli


class TestDusty_evolved_star_kit(unittest.TestCase):
    """Tests for `dusty_evolved_star_kit` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_000_something(self):
        """Test something."""

    def test_command_line_interface(self):
        """Test the CLI."""
        # runner = CliRunner()
        # result = runner.invoke(sed_fit.main)
        # assert result.exit_code == 0
        # assert 'Time:' in result.output
        #
        # help_result = runner.invoke(cli.main, ['--help'])
        # assert help_result.exit_code == 0
        # assert '--help  Show this message and exit.' in help_result.output



'''
Steve Goldman
Space Telescope Science Institute
May 17, 2018
sgoldman@stsci.edu

This script is for plotting the outputs of the sed_fitting script.
'''

target = dict(
    distance_in_kpc=50,
    assumed_gas_to_dust_ratio=200,
    input_unit='Jy'
)

# model options
# Built-in options: Zubko-Crich-aringer, Oss-Orich-bb, Oss-Orich-aringer, Crystalline-20-bb, corundum-20-bb
# Padova options: J400, J1000, H11, R12, R13
# Grams options:

fitting = dict(
    model_grid='Zubko-Crich-bb',
    wavelength_min=0.00001,
    wavelength_max=100,
)
output = dict(
    printed_output='True',

    output_unit='Wm^-2',
    # output_unit='Jy',

    create_figure='yes'
)
grids = ['Crystalline-20-bb',
         'H11-LMC',
         'H11-SMC',
         'J1000-LMC',
         'J1000-SMC',
         'Oss-Orich-aringer',
         'Oss-Orich-bb',
         'Zubko-Crich-aringer',
         'Zubko-Crich-bb',
         'arnold-palmer',
         'big-grain',
         'corundum-20-bb',
         'fifth-iron',
         'half-iron',
         'one-fifth-carbon',
         'grams-oxygen',
         'grams-carbon']



import astropy.units as u
import collections
import copy
import fnmatch
import importlib
import math
import matplotlib.pyplot as plt
import numpy as np
import os
import pdb
import pylab
import shutil
import time
from astropy.table import Table, Column, vstack
from astropy.units import astrophys
from desk import config
from numpy import ndarray

plt.rc('text', usetex=True)
plt.rcParams['font.family'] = 'serif'
plt.rcParams['mathtext.fontset'] = 'dejavuserif'


def create_par():
    counter = 0
    input_file = Table.read('fitting_plotting_outputs.csv')
    model = input_file['grid_name'][0]
    full_path = str(__file__.replace('parameter_ranges.py', ''))
    par = Table.read(full_path + 'models/' + model + '_outputs.csv')

    for i in par.colnames:
        if fnmatch.fnmatch(par[i].dtype.str, '<U*'):
            par.remove_column(i)
        else:
            par.rename_column(i, i.replace('_', ' '))

    if len(par.colnames) > 18:
        par = par[par.colnames[:17]]

    if len(par.colnames) < 8:
        fig, axs = plt.subplots(math.ceil(len(par.colnames)), 1, figsize=(8, 10))
        plt.subplots_adjust(wspace=0, hspace=0.5)
    else:
        fig, axs = plt.subplots(math.ceil(len(par.colnames)), 1, figsize=(5, 10))
        plt.subplots_adjust(wspace=0, hspace=1)

    axs = axs.ravel()

    axs[0].set_title('Model grid: ' + model, size=20)

    for col in par.colnames:
        par_min = np.min(par[col])
        par_max = np.max(par[col])
        axs[counter].scatter(par[col], [0] * len(par), marker='|', alpha=0.3, c='royalblue')
        range_min = par_min - ((par_max - par_min) * 0.1)
        range_max = par_max + (par_max - par_min) * 0.1
        if range_min != range_max:
            axs[counter].set_xlim(range_min, range_max)
        elif par_min == 0 and par_max == 0:
            axs[counter].set_xlim(-0.9, 0.9)
        else:
            axs[counter].set_xlim(range_min * 0.6, range_min * 1.4)
        # print(str(par_min) + ' : ' + str(par_max))
        axs[counter].set_ylabel(col)
        axs[counter].set_yticklabels([])
        axs[counter].set_yticks([])
        counter += 1
    fig.savefig('parameter_ranges_' + model + '.png', dpi=200, bbox_inches='tight')


if __name__ == '__main__':
    create_par()



import os
import pdb
import shutil
import sys
import tarfile
import urllib
from fnmatch import fnmatch

import numpy as np
from astropy.table import vstack, Table, Column
from tqdm import tqdm


def get_models(model_grid_name):
    def reporthook(blocknum, blocksize, totalsize):
        readsofar = blocknum * blocksize
        if totalsize > 0:
            percent = readsofar * 1e2 / totalsize
            s = "\r%5.1f%% %*d / %d" % (
                percent, len(str(totalsize)), readsofar, totalsize)
            sys.stderr.write(s)
            if readsofar >= totalsize:  # near the end
                sys.stderr.write("\n")
        else:  # total size is unknown
            sys.stderr.write("read %d\n" % (readsofar,))

    full_path = str(__file__.replace('get_remote_models.py', ''))
    grids = ['Crystalline-20-bb',
             'H11-SMC',
             'H11-LMC',
             'J1000-SMC',
             'J1000-LMC',
             'Oss-Orich-aringer',
             'Oss-Orich-bb',
             'Zubko-Crich-aringer',
             'Zubko-Crich-bb',
             'arnold-palmer',
             'big-grain',
             'corundum-20-bb',
             'fifth-iron',
             'half-iron',
             'one-fifth-carbon',
             'grams-oxygen',
             'grams-carbon'
             ]
    grid_csv_link = ['https://stsci.box.com/shared/static/jxtabj6h5zi7a8ggb0mbtag3v6b4b9i7.csv',
                     'https://stsci.box.com/shared/static/z0hnvo0hz0qxcwlcvbmf01avdjlhewah.csv',
                     'https://stsci.box.com/shared/static/j7f01dwbsbluq4htdwlsx3v0jknye883.csv',
                     'https://stsci.box.com/shared/static/o6dp1cyj86jw2ifuuco983kgb2uf16sf.csv',
                     'https://stsci.box.com/shared/static/fuhrcgo4vh4m7ca121p1f0vuss4vszcz.csv',
                     'https://stsci.box.com/shared/static/he92pt1yov18x6ska5f4f4q94gal6tr5.csv',
                     'https://stsci.box.com/shared/static/pnljezbu8c7rcyorb27yh6hylk5wy2fs.csv',
                     'https://stsci.box.com/shared/static/x9fg8lacbbiye591tz723ilcqgcomyjh.csv',
                     'https://stsci.box.com/shared/static/4b9rc9zl110khexeuo8thb8r58mrl73g.csv',
                     'https://stsci.box.com/shared/static/4gxmapkhml7lnxsiu9dzto8a7zwnmzd5.csv',
                     'https://stsci.box.com/shared/static/kfaq7jqt4t69vitf1cifk5gurx7lnogn.csv',
                     'https://stsci.box.com/shared/static/xf6rrfnp1jdin94iwhwb47cxoplu3aux.csv',
                     'https://stsci.box.com/shared/static/5zqshpzw748n0doykm7d5cxnlvhue4kk.csv',
                     'https://stsci.box.com/shared/static/x7328lqy8fyg19wswqwq4md4gm398ewi.csv',
                     'https://stsci.box.com/shared/static/8j73qttzj10eu5sct0ds0835uxbxu3s8.csv',
                     'https://stsci.box.com/shared/static/ldocm4yto4vtxfcg21o23a8uilb3a3do.csv',
                     'https://stsci.box.com/shared/static/iy6dsqrnhi96ecnlsozo1ljr5g8b1jwx.csv'
                     ]
    grid_fits_link = ['https://stsci.box.com/shared/static/1suiv6nqbc7yoqutli2q9q98ste31o1y.fits',
                      'https://stsci.box.com/shared/static/bq384o6m1xm41bkx7s8oloohqkd87hy8.fits',
                      'https://stsci.box.com/shared/static/faekj3usdme9go3lmynga3uk0oln8v3k.fits',
                      'https://stsci.box.com/shared/static/w0e3f1a0r886u17vjosk8lvqrnvnw6ce.fits',
                      'https://stsci.box.com/shared/static/e0nvigaed722a67043rvmqhjzkonttp4.fits',
                      'https://stsci.box.com/shared/static/60sns7ua91ixuz4rddul2ufrmc0pu5yo.fits',
                      'https://stsci.box.com/shared/static/hr113z7lgvggyybh9eygi6nj69gu1gzv.fits',
                      'https://stsci.box.com/shared/static/hvgfnug5xxcepcz083cnrizsukgddttw.fits',
                      'https://stsci.box.com/shared/static/c5gdw6o3b96kaphe07y459zd7ba6590f.fits',
                      'https://stsci.box.com/shared/static/b78ks7cjzenagoznqqrdfoyh6bonw2sy.fits',
                      'https://stsci.box.com/shared/static/lw6gp2s8surgbbtmcnyev673faxqg9h6.fits',
                      'https://stsci.box.com/shared/static/yq6pbvepyt8a420ealg6i111xc7hcty5.fits',
                      'https://stsci.box.com/shared/static/qjuq780xr0ihj9p909wmijwkpclu2njd.fits',
                      'https://stsci.box.com/shared/static/ht6edjsrupwhytwuwa9hjg53dof2zzy0.fits',
                      'https://stsci.box.com/shared/static/n1ng9f4s8s7ps0ah8vp24gk0fzsxp0aj.fits',
                      'https://stsci.box.com/shared/static/l1qayndbz40lwewvlpg1qxu22rs9f9xu.fits',
                      'https://stsci.box.com/shared/static/jho46vam6d0k9jwg78gxx4eh7emwx4oz.fits'
                      ]
    if any(ext in model_grid_name for ext in grids):
        match_index = [i for i, item in enumerate(grids) if model_grid_name == item][0]
        url_csv = grid_csv_link[match_index]
        url_fits = grid_fits_link[match_index]
    else:
        raise ValueError(
            'ERROR: Model name not an option. \nCurrent downloadable options: \n \t Zubko-Crich-aringer \n \t Oss-Orich-bb \n \t Oss-Orich-aringer \n \t Crystalline-20-bb \n \t corundum-20-bb \n \t arnold-palmer \n \t big-grains \n \t fifth-iron \n \t one-fifth-carbon')

    # \n Padova options: J400, J1000, H11, R12, R13'

    # Download files
    print(". . . Downloading model: " + model_grid_name + ' . . .')
    urllib.request.urlretrieve(url_csv, full_path + '/models/' + model_grid_name + '_outputs.csv', reporthook)
    urllib.request.urlretrieve(url_fits, full_path + '/models/' + model_grid_name + '_models.fits', reporthook)
    print("Download Complete!")


if __name__ == '__main__':
    get_models()



from __future__ import absolute_import

import csv
import glob
import copy
import importlib
import math
import os
import pdb
import time
from fnmatch import fnmatch
from multiprocessing import Pool, cpu_count
from multiprocessing import Process, Value, cpu_count

import astropy.units as u
import matplotlib.pyplot as plt
import numpy as np
from astropy.io import ascii
from astropy.table import Table, Column
from desk import config, plotting_seds, get_remote_models, parameter_ranges
from desk.parameter_ranges import create_par
from desk.plotting_seds import create_fig
from matplotlib import rc
from scipy import interpolate

importlib.reload(config)

'''
Steve Goldman
Space Telescope Science Institute
Nov 16, 2018
sgoldman@stsci.edu

This package takes a photometry or a spectrum (or spectra) in csv format and fits it/them with a grid of models. 
The grids used here are converted from output from the DUSTY code (Elitzur & Ivezic 2001, MNRAS, 327, 403) using
the other script dusty_to_grid.py. The code interpolates and trims a version of the data and calculates the least
squares value for each grid in the model and the data. The DUSTY outputs are then scaled and returned in files:
fitting_results.csv and fitting_plotting_output.csv (for plotting the results). An example plotting script has also
been provided.
'''


def grids():
    print('\nGrids:')
    for item in config.grids:
        print('\t' + str(item))
    print('\n')


def make_output_files_dusty():
    with open('fitting_results.csv', 'w') as f:
        f.write('source,L,vexp_predicted,teff,tinner,odep,mdot\n')
        f.close()
    with open('fitting_plotting_outputs.csv', 'w') as f:
        f.write('target_name,data_file,norm,index,grid_name,teff,tinner,odep\n')
        f.close()


def make_output_files_grams():
    with open('fitting_results.csv', 'w') as f:
        f.write('source,L,rin,teff,tinner,odep,mdot\n')
        f.close()
    with open('fitting_plotting_outputs.csv', 'w') as f:
        f.write('target_name,data_file,norm,index,grid_name,teff,tinner,odep\n')
        f.close()


def get_data(filename):
    """
    :param filename: filename of input data. Should be csv with Column 0: wavelength in um and Col 1: flux in Jy
    :return: two arrays of wavelength (x) and flux (y) in unit specified in config.py
    """
    global log_average_flux_wm2
    table = ascii.read(filename, delimiter=',')
    table.sort(table.colnames[0])
    x = np.array(table.columns[0])
    y = np.array(table.columns[1])
    index = np.where((x > config.fitting['wavelength_min']) & (x < config.fitting['wavelength_max']))
    x = x[index]
    y = y[index]
    y = y * u.Jy
    y = y.to(u.W / (u.m * u.m), equivalencies=u.spectral_density(x * u.um))
    log_average_flux_wm2 = np.log10(np.median(y).value)
    return x, np.array(y)


def find_closest(target_wave, model_wave):
    """
    :param target_wave: Target wavelength in um
    :param model_wave: model wavelength in um
    :return: a 1-D array of the closest data wavelength values, to the model wavelength values
    """
    idx = np.searchsorted(model_wave[0], target_wave[0])
    idx = np.clip(idx, 1, len(model_wave[0]) - 1)
    left = model_wave[0][idx - 1]
    right = model_wave[0][idx]
    idx -= target_wave[0] - left < right - target_wave[0]
    closest_data_flux = model_wave[1][idx]
    return closest_data_flux


def least2(data, model_l2):
    return np.nansum(np.square(model_l2 - data))


def trim(data, model_trim):
    """
    :param data: input data in tuple of x and y arrays
    :param model_trim: input model
    :return: trims motimedel to wavelength range of data
    """
    indexes = np.where(np.logical_and(model_trim[0] >= np.min(data[0]), model_trim[0] <= np.max(data[0])))
    return np.vstack([model_trim[0][indexes], model_trim[1][indexes]])


def fit_norm(data, norm_model):
    """
    :param data: input data in tuple of x and y arrays
    :param norm_model: closest wavelength values to data in 1-D array (from trim)
    :return: trimmed model in 2 column np.array
    """
    global trials
    stats = []
    # normalization range
    # trials = np.linspace(config.fitting['min_norm'], config.fitting['max_norm'], config.fitting['ntrials'])
    trials = np.logspace(log_average_flux_wm2 - 2, log_average_flux_wm2 + 2, number_of_tries)
    for t in trials:
        stat = least2(data[1], norm_model * t)
        stats.append(stat)
    return stats


# for each target, fit spectra with given models (.fits file)
def sed_fitting(target):
    stat_values = []
    raw_data = get_data(target)  # gets target data
    # model_x = grid_dusty[0][0][:]  # gets model wavelengths
    for model in np.array(grid_dusty):
        trimmed_model = trim(raw_data, model)  # gets fluxes for corresponding wavelengths of data and models
        matched_model = find_closest(raw_data, trimmed_model)
        stat_values.append(fit_norm(raw_data, matched_model))
    stat_array = np.vstack(stat_values)
    argmin = np.argmin(stat_array)
    model_index = argmin // stat_array.shape[1]
    trial_index = argmin % stat_array.shape[1]
    target_name = (target.split('/')[-1][:15]).replace('IRAS-', 'IRAS ')

    def dusty_fit():
        # calculates luminosity and scales outputs
        luminosity = int(np.power(10.0, distance_norm - math.log10(trials[trial_index]) * -1))
        scaled_vexp = float(grid_outputs[model_index]['vexp']) * (luminosity / 10000) ** 0.25
        scaled_mdot = grid_outputs[model_index]['mdot'] * ((luminosity / 10000) ** 0.75) * (
                config.target['assumed_gas_to_dust_ratio'] / 200) ** 0.5

        teff = int(grid_outputs[model_index]['teff'])
        tinner = int(grid_outputs[model_index]['tinner'])
        odep = grid_outputs[model_index]['odep']

        # creates output file
        latex_array = [target_name, luminosity, np.round(scaled_vexp, 1),
                       teff, tinner, odep, "%.3E" % float(scaled_mdot)]

        plotting_array = [target_name, target, trials[trial_index], model_index, model_grid, teff, tinner, odep]

        # printed output
        if config.output['printed_output'] == 'True':
            print()
            print()
            print(('             Target: ' + target_name + '        ' + str(counter.value + 1) + '/' + str(
                number_of_targets)))
            print('-------------------------------------------------')
            print(("Luminosity\t\t\t|\t" + str(round(luminosity))))
            print(("Optical depth\t\t\t|\t" + str(round(grid_outputs[model_index]['odep'], 3))))
            print(("Expansion velocity (scaled)\t|\t" + str(round(scaled_vexp, 2))))
            print(("Gas mass loss (scaled)\t\t|\t" + str("%.2E" % float(scaled_mdot))))
            print('-------------------------------------------------')
        with open('fitting_results.csv', 'a') as f:
            writer = csv.writer(f, delimiter=',', lineterminator='\n')
            writer.writerow(np.array(latex_array))
            f.close()

        with open('fitting_plotting_outputs.csv', 'a') as f:
            writer = csv.writer(f, delimiter=',', lineterminator='\n')
            writer.writerow(np.array(plotting_array))
            f.close()
        counter.value += 1

    def grams_fit():
        # pdb.set_trace()
        luminosity = grid_outputs[model_index]['lum'] * ((distance_value / 50) ** 2)
        teff = grid_outputs[model_index]['teff']
        tinner = grid_outputs[model_index]['tinner']
        odep = grid_outputs[model_index]['odep']
        mdot = grid_outputs[model_index]['mdot'] * (distance_value / 50)
        rin = grid_outputs[model_index]['rin'] * (distance_value / 50)
        # creates output file
        latex_array = [target_name, luminosity, rin, teff, tinner, odep, "%.3E" % float(mdot)]

        plotting_array = [target_name, target, trials[trial_index], model_index, model_grid, teff, odep]
        if config.output['printed_output'] == 'True':
            print()
            print()
            print(('             Target: ' + target_name + '        ' + str(counter.value + 1) + '/' + str(
                number_of_targets)))
            print('-------------------------------------------------')
            print(("Luminosity\t\t\t|\t" + str(round(luminosity))))
            print(("Optical depth\t\t\t|\t" + str(round(grid_outputs[model_index]['odep'], 3))))
            print(("Inner Radius\t\t\t|\t" + str(rin)))
            print(("Dust production rate \t\t|\t" + str("%.2E" % float(mdot))))
            print('-------------------------------------------------')
        with open('fitting_results.csv', 'a') as f:
            writer = csv.writer(f, delimiter=',', lineterminator='\n')
            writer.writerow(np.array(latex_array))
            f.close()

        with open('fitting_plotting_outputs.csv', 'a') as f:
            writer = csv.writer(f, delimiter=',', lineterminator='\n')
            writer.writerow(np.array(plotting_array))
            f.close()
        counter.value += 1

    if fnmatch(model_grid, 'grams*'):
        grams_fit()
    else:
        dusty_fit()


def fit(source='default', distance=config.target['distance_in_kpc'], grid=config.fitting['model_grid']):
    """
    :param source: Name of target in array of strings (or one string)
    :param distance: distance to source(s) in kiloparsecs
    :param grid: Name of model grid
    :return:
    """
    # set variables
    global model_grid
    global counter
    global grid_dusty
    global grid_outputs
    global distance_norm
    global distance_value
    global full_path
    global files
    global number_of_tries
    global number_of_targets
    start = time.time()
    counter = Value('i', 0)
    number_of_tries = 5
    # normalization calculation
    # solar constant = 1379 W
    # distance to sun in kpc 4.8483E-9
    distance_value = float(copy.copy(distance))
    distance_norm = math.log10(((float(distance) / 4.8482E-9) ** 2) / 1379)
    full_path = str(__file__.replace('sed_fit.py', ''))

    # User input for models
    if grid == 'carbon':
        model_grid = 'Zubko-Crich-bb'
    elif grid == 'oxygen':
        model_grid = 'Oss-Orich-bb'
    else:
        if grid in config.grids:
            model_grid = grid
        else:
            raise ValueError(
                '\n\nUnknown grid. Please make another model selection.\n\n To see options use: desk grids\n')

    # check if models exist
    csv_file = full_path + 'models/' + model_grid + '_outputs.csv'
    fits_file = full_path + 'models/' + model_grid + '_models.fits'
    if os.path.isfile(csv_file) and os.path.isfile(fits_file):
        print('You already have the models!')
        print('Great job')

    else:
        user_proceed = input('Models not found locally, download the models [y]/n?: ')
        if user_proceed == 'y' or user_proceed == '':
            get_remote_models.get_models(grid)
        elif user_proceed == 'n':
            raise ValueError('Please make another model selection')
        elif user_proceed != 'y' and user_proceed != 'n':
            raise ValueError('Invalid selection')

    # gets models
    grid_dusty = Table.read(fits_file)
    grid_outputs = Table.read(csv_file)

    # Model grid equal lengths check
    if len(grid_dusty) == len(grid_outputs):
        pass
    else:
        raise ValueError("Model grid input error: mismatch in model spectra and model output")

    # creates correct results files
    if fnmatch(model_grid, 'grams*'):
        make_output_files_grams()
    else:
        make_output_files_dusty()

    # SED FITTING ###############################

    if source == 'default':
        source = full_path + 'put_target_data_here/'
    if fnmatch(source, '*.csv'):
        number_of_targets = 1
        sed_fitting(source)
    elif os.path.isdir(source):
        source_dir = (source + '/').replace('//', '/')
        if glob.glob(source_dir + '/*.csv'):
            files = os.listdir(source)
            files = glob.glob(source + '/' + '*.csv')
            number_of_targets = len(files)
            with Pool(processes=cpu_count() - 1) as pool:
                pool.map(sed_fitting, [target_string for target_string in files])
        else:
            raise ValueError('\n\n\nERROR: No .csv files in that directory. Please make another selection.\n\n')
    else:
        raise ValueError('\n\n\nError: Not a .csv file. Please make another selection.\n\n')

    # creating figures
    if config.output['create_figure'] == 'yes':
        print('\n. . . Creating SED figure . . . . . . . . . . . .')
        create_fig()  # runs plotting script
    else:
        print('No figure created. To automatically generate a figure change the ' +
              '"create_figure" variable in the config.py script to "yes".')

    if not os.path.isfile("parameter_ranges_" + config.fitting['model_grid'] + ".png"):
        print('. . . Creating parameter range figure . . . . . .')
        create_par()

    end = time.time()
    print()
    print('Time: ' + str("%.2f" % float((end - start) / 60)) + ' minutes')


if __name__ == '__main__':
    fit()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = ['Click>=6.0', 'tqdm', 'scipy']

setup_requirements = ['numpy']

test_requirements = ['scipy']

setup(
    author="Steven R. Goldman",
    author_email='sgoldman@stsci.edu',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    description="The DESK is an SED-fitting python scripts for fitting data from evolved stars",
    entry_points={'console_scripts': ['desk = desk.main:main']},
    install_requires=requirements,
    license="BSD license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='Dusty_Evolved_Star_Kit',
    name='desk',
    packages=find_packages(include=['desk']),
    package_data={'project': ['desk/models/*', 'desk/put_target_data_here/*']},
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/s-goldman/Dusty_Evolved_Star_Kit',
    version='1.6.1',
    zip_safe=False,
)
